<?php
/**
 * jDiction database class
 *
 * extends the current joomla database class with translation functions
 *
 * @package jDiction
 * @link http://joomla.itronic.at
 * @copyright  Copyright (C) 2011 ITronic Harald Leithner. All rights reserved.
 * @license GNU General Public License v3
 *
 * This file is part of jDiction.
 *
 * jDiction is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * jDiction is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with jDiction.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
defined('_JEXEC') or die;

jimport('jdiction.jdiction');

/**
 * Database connector class
 *
 * @package  jDiction
 */
class JdDatabaseViewXML extends JdDatabaseAdapter {

  /** @var array of tables with translations */
  //protected $jd_tables = Array();

  /** @var object JDispatcher with jdiction plugins */
  protected $jd_dispatcher;

  /** @var bool should we translate */
  protected $jd_active = true;

  /** @var int the language we should translate to */
  protected $jd_language;

  /** @var array the fields of a query to translate */
  protected $jd_metadata;

  /** @var object the jDiction object */
  protected $jd;

  /** @var array Temporary stored queries */
  protected $stash = array();

  /**
   * Database object constructor
   *
   * @param  array $options List of options used to configure the connection
   * @since  1.5
   */
  public function __construct($options) {

    $this->jd = jDiction::getInstance();

    //this is not possible because we get initialised in getApplication
    //$app = JFactory::getApplication();
    //$clientid = $app->getClientid();

    //workaround to detect admin
    if (strtolower(substr(JPATH_BASE, -13)) == 'administrator') {
      $clientid = 1;
    } else {
      $clientid = 0;
    }
    if ($clientid == 1) {
      $this->jd_active = false;
    }

    parent::__construct($options);
  }


  public function __sleep() {
    // Workaround for clone in JDatabasequery clone function but maybe a good idea anyway
    $safe = array_keys(get_object_vars($this));
    /* does not work as expected
    if (method_exists(parent, '__sleep')) {
      $parentsafe = parent::__sleep();
      $safe = $safe + $parentsafe;
    };
    */
    $safe = array_flip($safe);
    unset($safe['jd']);
    $safe = array_flip($safe);

    return $safe;
  }

  public function __wakeup() {
    // Workaround for clone in JDatabasequery clone function but maybe a good idea anyway

    /* does not work as expected
    if (method_exists(parent, '__wakeup')) {
      parent::__wakeup();
    };
    */
    $this->jd = jDiction::getInstance();

  }
  /** +Database functions */


  /**
   * Get the number of rows returned by the most recent query
   *
   * @param  object $cur Database resource
   * @return  int    The number of rows
   */
  public function getNumRows($cur = null) {
    $count = parent::getNumRows($cur);
    if (!$this->translateQuery()) {
      return $count;
    }
    // atm we only make 1to1 translation without the possibility to deactivte a item for language
    // this possible with joomla it self by set the item to a specific language
    return $count;
  }

  /**
   * This method loads the first field of the first row returned by the query.
   *
   * @return  mixed  The value returned in the query or null if the query failed.
   */
  public function loadResult() {
    if (!$this->translateQuery()) {
      $result = parent::loadResult();
      return $result;
    }

    $result = $this->loadTranslation(-1, 'NUM');
    if (!is_null($result)) {
      return $result[0];
    } else {
      return null;
    }

  }

  /**
   * Method to get an array of values from the <var>$offset</var> field in each row of the result set from
   * the database query.
   *
   * @param   integer $offset The row offset to use to build the result array.
   *
   * @return  mixed    The return value or null if the query failed.
   *
   * @since   11.1
   * @throws  JDatabaseException
   */
  public function loadColumn($offset = 0) {
    if (!$this->translateQuery()) {
      $result = parent::loadColumn($offset);
      return $result;
    }

    // Initialise variables.
    $array = array();

    $result = $this->loadTranslation(null, 'NUM');
    foreach ($result as &$row) {
      $array[] = $row[$offset];
    }

    return $array;
  }

  /**
   * Fetch a result row as an associative array
   *
   * @return array  first result row as associatve array
   */
  public function loadAssoc() {
    if (!$this->translateQuery()) {
      $result = parent::loadAssoc();
      return $result;
    }

    $result = $this->loadTranslation(-1, 'ASSOC');

    return $result;
  }

  /**
   * Load a assoc list of database rows
   *
   * @param  string $key The field name of a primary key
   * @param  string $column An optional column name. Instead of the whole row, only this column value will be in the return array.
   * @return  array  If <var>key</var> is empty as sequential list of returned records.
   */
  public function loadAssocList($key = null, $column = null) {
    if (!$this->translateQuery()) {
      $result = parent::loadAssocList($key, $column);
      return $result;
    }

    $result = $this->loadTranslation($key, 'ASSOC');

    if ($column) {
      $array = array();
      foreach ($result as $k => $row) {
        $array[$k] = $row[$column];
      }
      $result = $array;
    }

    return $result;
  }

  /**
   * This global function loads the first row of a query into an object
   *
   * @param  string $className The name of the class to return (stdClass by default).
   *
   * @return  object
   */
  public function loadObject($className = 'stdClass') {
    if (!$this->translateQuery()) {
      $result = parent::loadObject($className);
      return $result;
    }

    $result = $this->loadTranslation(-1, 'ASSOC');
    $result = JArrayHelper::toObject($result, $className);
    return $result;

  }


  /**
   * Load a list of database objects
   *
   * If <var>key</var> is not empty then the returned array is indexed by the value
   * the database key.  Returns <var>null</var> if the query fails.
   *
   * @param  string $key The field name of a primary key
   * @param  string $className The name of the class to return (stdClass by default).
   *
   * @return  array  If <var>key</var> is empty as sequential list of returned records.
   */
  public function loadObjectList($key = null, $className = 'stdClass') {
    if (!$this->translateQuery()) {
      $result = parent::loadObjectList($key, $className);
      return $result;
    }

    $result = $this->loadTranslation($key, 'ASSOC');

    if (is_array($result)) {
      $array = array();
      foreach ($result as $k => $row) {
        $array[$k] = JArrayHelper::toObject($row, $className);
      }
    } else {
      return $result;
    }

    return $array;
  }


  /**
   * Load the first row returned by the query
   *
   * @return  mixed  The first row of the query.
   */
  public function loadRow() {
    $result = parent::loadRow();
    if (!$this->translateQuery()) {
      return $result;
    }

    $result = $this->loadTranslation(-1, 'NUM');
    return $result;
  }

  /**
   * Load a list of database rows (numeric column indexing)
   *
   * If <var>key</var> is not empty then the returned array is indexed by the value
   * the database key.  Returns <var>null</var> if the query fails.
   *
   * @param  string $key The field name of a primary key
   * @return  array
   */
  public function loadRowList($key = null) {
    if (!$this->translateQuery()) {
      $result = parent::loadRowList($key);
      return $result;
    }

    return $this->loadTranslation($key, 'NUM');
  }

  /**
   * Load the next row returned by the query.
   *
   * @return  mixed  The result of the query as an array, false if there are no more rows, or null on an error.
   *
   * @since  1.6.0
   */
  public function loadNextRow() {
    $result = parent::loadNextRow();

    if (!$this->translateQuery()) {
      return $result;
    }

    if ($result === false) {
      return false;
    }
    $result = array($result);
    $this->jd->translate($result, $this->getLanguage(), $this->jd_metadata, 'NUM');

    return $result[0];
  }

  /**
   * Load the next row returned by the query.
   *
   * @param  string $className The name of the class to return (stdClass by default).
   *
   * @return  mixed  The result of the query as an object, false if there are no more rows, or null on an error.
   *
   * @since  1.6.0
   */
  public function loadNextObject($className = 'stdClass') {
    if (!$this->translateQuery()) {
      return parent::loadNextObject($className);
    }

    $result = $this->loadNextRow();
    if ($result === false) {
      return false;
    }

    $result = JArrayHelper::toObject($result, $className);

    return $result;
  }

  /**
   * Load a list of database rows and translate it
   *
   * If <var>key</var> is not empty then the returned array is indexed by the value
   * the database key.  Returns <var>null</var> if the query fails.
   *
   * @param  string|int $key The field name or the field positon of a primary key or -1
   * for a single row
   * @param string $resulttype The result type of the array (ASSOC, NUM, BOTH)
   * @return  array
   */
  protected function loadTranslation($key = null, $resulttype = 'BOTH') {
    if (is_numeric($key) && $key < 0) {
      $result = parent::loadRow();
    } else {
      //we handle $key at jDiction::translate
      $result = parent::loadRowList(null);
    }

    $this->translate($result, $this->getLanguage(), $this->jd_metadata, $resulttype, $key);

    return $result;
  }



  /** -Database functions */


  /**
   *
   * Returns if we should translate by hand or not
   * @return bool translate this query
   */
  protected function translateQuery($fallback = true) {
    $translate = false;

    if (!empty($_REQUEST['jDictionDebug'])) JLog::add('Query: ' . str_replace("\n", '\n', $this->sql), JLog::DEBUG, 'jdiction');
    if (!empty($_REQUEST['jDictionDebug']) && $_REQUEST['jDictionDebug'] > 1) JLog::add('Backtrace: ' . @json_encode(debug_backtrace()), JLog::DEBUG, 'jdiction');

    $this->jd->initialise();

    /* Check if translation is active */
    if (!$this->jd_active) {
      if (!empty($_REQUEST['jDictionDebug'])) JLog::add('Query not translated (!status): ' . str_replace("\n", '\n', $this->sql), JLog::DEBUG, 'jdiction');
      return false;
    }

    /* We can't translate if we don't have a language */
    if (is_null($this->jd_language)) {
      if (!empty($_REQUEST['jDictionDebug'])) JLog::add('Query not translated (!lang): ' . str_replace("\n", '\n', $this->sql), JLog::DEBUG, 'jdiction');
      return false;
    }

    /* Check if jDiction is ready */
    if (!$this->jd->getStatus()) {
      if (!empty($_REQUEST['jDictionDebug'])) JLog::add('Query not translated (!jd_active): ' . str_replace("\n", '\n', $this->sql), JLog::DEBUG, 'jdiction');
      return false;
    }

    /* We only translate SELECT queries */
    if (strtoupper(substr(ltrim($this->sql), 0, 6)) != 'SELECT') {
      if (!empty($_REQUEST['jDictionDebug'])) JLog::add('Query not translated (!select): ' . str_replace("\n", '\n', $this->sql), JLog::DEBUG, 'jdiction');
      return false;
    }

    /* We don't translate jDiction tables */
    if (strpos($this->sql, '#__jd_') !== false) {
      if (!empty($_REQUEST['jDictionDebug'])) JLog::add('Query not translated (jdtable): ' . str_replace("\n", '\n', $this->sql), JLog::DEBUG, 'jdiction');
      return false;
    }

    /* We don't translate queries with / * NOTRANSLATION * / in it */
    if (strpos($this->sql, '/* NOTRANSLATION */') !== false) {
      if (!empty($_REQUEST['jDictionDebug'])) JLog::add('Query not translated (notrans): ' . str_replace("\n", '\n', $this->sql), JLog::DEBUG, 'jdiction');
      return false;
    }

    /* We don't translate the default language - CAUTION this check has to be done after we filtered #__languages queries */
    if ($this->jd_language == $this->jd->getDefaultLanguage()->lang_id) {
      if (!empty($_REQUEST['jDictionDebug'])) JLog::add('Query not translated (default lang): ' . str_replace("\n", '\n', $this->sql), JLog::DEBUG, 'jdiction');
      return false;
    }


    //search for a supported table
    $tables = $this->jd->getTable();
    foreach ($tables as $table) {
      // we only need to touch the result if we have a json field
      if (strpos($this->sql, $table->name) !== false) {
        if ($fallback) {
          if ($table->hasjson) {
            $translate = true;
            break;
          }
        } else {
          $translate = true;
          break;
        }
      }
    }

    if (!empty($_REQUEST['jDictionDebug'])) {
      if ($translate) {
        JLog::add('Query translated: ' . str_replace("\n", '\n', $this->sql), JLog::DEBUG, 'jdiction');
      } else {
        JLog::add('Query not translated (not needed): ' . str_replace("\n", '\n', $this->sql), JLog::DEBUG, 'jdiction');
      }
    }

    return $translate;
  }

  /**
   * Execute the query
   *
   * @return  mixed  A database resource if successful, FALSE if not.
   */
  public function execute() {

    // Check if we want and are ready to translate this query
    if ($this->translateQuery(false)) {
      $origsql      = $this->sql;
      $origlimit    = $this->limit;
      $origoffset   = $this->offset;
      $sql          = $this->replaceTablesWithViews($origsql);
      $this->limit  = $origlimit;
      $this->offset = $origoffset;
      $this->sql    = $sql;
      try {
        parent::execute();
        //restore original query
        $this->sql = $origsql;
      } catch (RuntimeException $e) {
        // there was a Problem with the query so we fallback to the unmodified
        // version and try again
        $this->sql = $origsql;
        parent::execute();
      }
      $this->collectTranslationTables();
    } else {
      parent::execute();
    }

    return $this->cursor;
  }

  protected function replaceTablesWithViews($sql) {

    // Load profiler
    $profiler = JProfiler::getInstance('jDiction');

    // Initialize variables.
    $tableList = $this->jd->getTable();
    $lang_id   = $this->getLanguage();
    $allTables = $this->getTableList();

    //test for non jdatabasequery
    $sql = trim((string)$sql);

    $origTablePrefix = $this->tablePrefix;

    foreach ($tableList as $k => $table) {
      $view   = str_replace('#__', '#__jd_' . $lang_id . '_', $table->name);
      $oldSql = $sql;
      $sql    = $this->replaceTableWithView($sql, $table->name, $view);
      if ($oldSql != $sql) {
        if (!in_array($this->replacePrefix($view), $allTables)) {
          static $createdTranslationViews = false;
          if (!$createdTranslationViews) {
            jimport('jdiction.helpers.viewxml');
            jDictionViewXmlHelper::createTranslationViews();
            $createdTranslationViews = true;
          }
        }
      }
    }


    $this->tablePrefix = $origTablePrefix;

    $profiler->mark('End replaceTablesWithView');

    return $sql;
  }

  /**
   * This function replaces a string identifier <var>$prefix</var> with the string held is the
   * <var>tablePrefix</var> class variable.
   *
   * @param   string $sql The SQL statement to prepare.
   * @param   string $prefix The common table prefix.
   *
   * @return  string  The processed SQL statement.
   *
   * @since   11.1
   */
  public function replaceTableWithView($sql, $table, $view) {
    $startPos = 0;
    $literal  = '';

    $sql = trim($sql);
    $n   = strlen($sql);

    while ($startPos < $n) {
      $ip = strpos($sql, $table, $startPos);

      if ($ip === false) {
        break;
      }

      $j = strpos($sql, "'", $startPos);
      $k = strpos($sql, '"', $startPos);

      if (($k !== false) && (($k < $j) || ($j === false))) {
        $quoteChar = '"';
        $j         = $k;
      } else {
        $quoteChar = "'";
      }

      if ($j === false) {
        $j = $n;
      }

      $part = substr($sql, $startPos, $j - $startPos);
      $part = preg_replace('/(' . preg_quote($table) . ')(\s|\t|`|\n|$)/', $view . '\2', $part);

      $literal .= $part;

      $startPos = $j;

      $j = $startPos + 1;

      if ($j >= $n) {
        break;
      }

      // Quote comes first, find end of quote
      while (true) {
        $k       = strpos($sql, $quoteChar, $j);
        $escaped = false;

        if ($k === false) {
          break;
        }

        $l = $k - 1;

        while ($l >= 0 && $sql{$l} == '\\') {
          $l--;
          $escaped = !$escaped;
        }

        if ($escaped) {
          $j = $k + 1;
          continue;
        }

        break;
      }

      if ($k === false) {
        // Error in the query - no end quote; ignore it
        break;
      }

      $literal .= substr($sql, $startPos, $k - $startPos + 1);
      $startPos = $k + 1;
    }

    if ($startPos < $n) {
      $literal .= substr($sql, $startPos, $n - $startPos);
    }

    return $literal;
  }

  /**
   * Assumes database collation in use by sampling one text field in one table
   *
   * @return  string  Collation in use
   */
  public function getCollation() {
    //proxy without translation
    $old    = $this->setTranslate(false);
    $result = parent::getCollation();
    $this->setTranslate($old);
    return $result;
  }

  /**
   * Description
   *
   * @return  array  A list of all the tables in the database
   */
  public function getTableList() {
    //proxy without translation
    $old    = $this->setTranslate(false);
    $result = parent::getTableList();
    $this->setTranslate($old);
    return $result;
  }

  /**
   * Shows the CREATE TABLE statement that creates the given tables
   *
   * @param  array|string A table name or a list of table names
   * @return  array A list the create SQL for the tables
   */
  public function getTableCreate($tables) {
    //proxy without translation
    $old    = $this->setTranslate(false);
    $result = parent::getTableCreate($tables);
    $this->setTranslate($old);
    return $result;
  }

  /**
   * Retrieves field information about the given tables.
   *
   * @param   string $table The name of the database table.
   * @param   boolean $typeOnly True (default) to only return field types.
   *
   * @return  array  An array of fields by table.
   *
   * @since   11.1
   * @throws  JDatabaseException
   */
  public function getTableColumns($tables, $typeonly = true) {
    //proxy without translation
    $old    = $this->setTranslate(false);
    $result = parent::getTableColumns($tables, $typeonly);
    $this->setTranslate($old);
    return $result;
  }

  /**
   * Set the target language to translate to
   *
   * @param int $language Language id to translate, null for default
   */
  public function setLanguage($language) {

    static $languageCache;
    static $defaultLanguage = null;
    if (is_null($defaultLanguage)) {
      $defaultLanguage = JComponentHelper::getParams('com_languages')->get('site', 'en-GB');
    }

    //load language id from #__languages table
    if (!is_array($languageCache)) {
      $query = $this->getQuery(true);
      $query->select('lang_id, lang_code');
      $query->from('#__languages');
      $this->setQuery($query);
      $old           = $this->setTranslate(false);
      $languageCache = $this->loadAssocList('lang_code', 'lang_id');
      $this->setTranslate($old);
    }

    if ($this->jd_language) {
      $return = $this->jd_language;
    } else {
      $return = $languageCache[$defaultLanguage];
    }

    if ((int)$language > 0) {
      $this->jd_language = $language;
    } else {
      if (array_key_exists($language, $languageCache)) {
        $this->jd_language = $languageCache[$language];
      } else {
        throw new Exception(500, 'Could not find language. (' . $language . ')');
      }
    }

    if ($this->jd_language == $languageCache[$defaultLanguage]) {
      $this->setTranslate(false);
    } else {
      $this->setTranslate(true);
    }

    return $return;
  }

  /**
   * Returns the language we translate to
   *
   * @return bool the language id
   */
  public function getLanguage() {
    if (!$this->jd_language) {
      $lang = JFactory::getLanguage();
      $this->setLanguage($lang->getTag());

      if ($lang->getTag() == $this->jd->getDefaultLanguage()->lang_code) {
        $this->setTranslate(false);
      } else {
        $this->setTranslate(true);
      }

    }
    return $this->jd_language;
  }

  /**
   * Activate or deactive the translation module
   *
   * @param bool $status Should we translate
   */
  public function setTranslate($status) {
    $old             = $this->jd_active;
    $this->jd_active = $status;
    return $old;
  }

  /**
   * Returns the translation state
   *
   * @return bool the state of the translation module
   */
  public function getTranslate() {
    return $this->jd_active;
  }

  /**
   * Copies the current query to stash
   */
  public function stashQuery() {
    array_unshift($this->stash, $this->sql);
  }

  /**
   * moves the last query from the stash to the active current query
   */
  public function unstashQuery() {
    $this->sql = array_shift($this->stash);
  }

  /** +translation functions */

  protected function replaceViewPrefix($table) {
    return preg_replace('/(#__jd_\d+_)/', '#__', $table);
  }

  /**
   * translate content
   */
  private function translate(&$rows, $language, &$metadata, $resulttype = 'BOTH', $key = null) {

    if (!$this->jd->getStatus()) {
      return false;
    }

    /* If we have no result we exit here */
    if (empty($rows)) {
      return false;
    }

    $aliaskey  = array();
    $newrows   = array();
    $alltables = $this->jd->getTable();

    //find primary key for each table alias
    foreach ($metadata['tables'] as $tablealias => $table) {
      foreach ($table['fields'] as $fieldid => $field) {
        if (isset($field->orgname)) {
          $fieldName = $field->orgname;
        } else {
          $fieldName = $field->name;
        }
        if (isset($alltables[$metadata['tablealias'][$tablealias]]) && $fieldName == $alltables[$metadata['tablealias'][$tablealias]]->key) {
          $aliaskey[$tablealias] = $fieldid;
        }
      }
    }

    // if the key is not ava
    if (empty($aliaskey)) {
      return false;
    }

    if (is_numeric($key) && $key < 0) {
      $rows = array($rows);
    }

    //translate all rows
    if (is_array($rows) && count($rows)) {
      // PRELOAD CACHE
      foreach ($metadata['tables'] as $tablealias => $table) {
        if (array_key_exists($metadata['tablealias'][$tablealias], $alltables)) {
          $ids = array();
          foreach ($rows as $row) {
            if (!empty($row[$aliaskey[$tablealias]])) {
              $ids[] = $row[$aliaskey[$tablealias]];
            }
          }
          $this->getTranslation($metadata['tablealias'][$tablealias], $ids, $language);
        }
      }

      foreach ($rows as $rowkey => &$row) {
        $targetkey = $rowkey;
        foreach ($metadata['tables'] as $tablealias => $table) {
          if (array_key_exists($metadata['tablealias'][$tablealias], $alltables) && $alltables[$metadata['tablealias'][$tablealias]]->hasjson) {
            $translation = $this->getTranslation($metadata['tablealias'][$tablealias], $row[$aliaskey[$tablealias]], $language);
          } else {
            $translation = false;
          }

          /* Unset this row if it has state -3 (hide) */
          if ($translation) {
            if ($translation['__metadata']->state == -3) {
              unset($rows[$rowkey]);
              continue 2;
            }
          }

          foreach ($table['fields'] as $fieldid => $field) {
            if (isset($field->orgname)) {
              $fieldName = $field->orgname;
            } else {
              $fieldName = $field->name;
            }

            if ($translation) {
              if (array_key_exists($fieldName, $translation)) {
                //used for params
                // @todo rework with recrusive version
                if (is_array($translation[$fieldName])) {
                  //works only for json atm
                  $jsontmp = json_decode($row[$fieldid], true);
                  foreach ($translation[$fieldName] as $subkey => $subvalue) {
                    $jsontmp[$subkey] = $subvalue;
                  }
                  $row[$fieldid] = json_encode($jsontmp);
                } else {
                  // we use views in this database driver to do normal translations
                }
              }
            }

            if ($key) {
              if (is_numeric($key)) {
                if ($key == $fieldid) {
                  $targetkey = $row[$fieldid];
                }
              } elseif (is_string($key)) {
                if ($key == $field->name) {
                  $targetkey = $row[$fieldid];
                }
              }
            }
          }

          //second turn to copy table to correct key and fieldnames
          foreach ($table['fields'] as $fieldid => $field) {
            switch (strtoupper($resulttype)) {
              case 'BOTH':
                $newrows[$targetkey][$fieldid]     = $row[$fieldid];
                $newrows[$targetkey][$field->name] = $row[$fieldid];
                break;
              case 'NUM':
                $newrows[$targetkey][$fieldid] = $row[$fieldid];
                break;
              case 'ASSOC':
                $newrows[$targetkey][$field->name] = $row[$fieldid];
                break;
            }
          }
        }
      }
    }
    unset($row);

    if (is_numeric($key) && $key < 0) {
      $rows = current($newrows);
    } else {
      $rows = $newrows;
    }
    return true;
  }

  private function getTranslation($reftable, $refids, $language) {

    $alltables = $this->jd->getTable();

    static $cache = array();

    if (array_key_exists($reftable, $cache)) {
      if (array_key_exists($language, $cache[$reftable])) {
        if (is_array($refids)) {
          foreach ($refids as $key => $refid) {
            if (array_key_exists($refid, $cache[$reftable][$language])) {
              unset($refids[$key]);
            } else {
              if ($alltables[$reftable]->cache->fullfetch) {
                unset($refids[$key]);
              }
            }
          }
        } else {
          if (array_key_exists($refids, $cache[$reftable][$language])) {
            return $cache[$reftable][$language][$refids];
          } else {
            // We don't do a full fetch if we don't translate the current language (mainly used for #__menu)
            if ($alltables[$reftable]->cache->fullfetch && $language == $this->getCurrentLanguage()->lang_id) {
              return false;
            }
          }
        }
      }
    }

    // check if we should precache some translations and return if no ids are given
    if (is_array($refids) && empty($refids)) {
      return false;
    }

    //save current state
    $this->stashQuery();

    $query = $this->getQuery(true);
    $query->select('*');
    $query->from('#__jd_store');
    $query->where('idLang=' . $this->quote($language));
    $query->where('referenceTable=' . $this->quote($reftable));

    // We don't do a full fetch if we don't translate the current language (mainly used for #__menu)
    if (!$alltables[$reftable]->cache->fullfetch || $language != $this->jd->getCurrentLanguage()->lang_id) {
      if (is_array($refids)) {
        $query->where('idReference in (' . implode(',', $refids) . ')');
      } else {
        if ($refids == '') {
          return false;
        }
        $query->where('idReference=' . $this->quote($refids));
      }
    }

    $this->setQuery($query);
    $translations = $this->loadObjectList();

    //restore old state
    $this->unstashQuery();

    if (is_array($translations)) {
      foreach ($translations as $translation) {
        $tmp = jDictionTranslationHelper::decodeTranslation($translation->value);
        if (empty($tmp)) {
          continue;
        }
        $cache[$reftable][$language][$translation->idReference] = $tmp;
        unset($translation->value);

        $cache[$reftable][$language][$translation->idReference]['__metadata'] = $translation;
      }
    }

    // Array requests are for caching only
    if (is_array($refids)) {
      foreach ($refids as $refid) {
        if (!isset($cache[$reftable][$language][$refid])) {
          $cache[$reftable][$language][$refid] = false;
        }
      }
      return false;
    } else {
      if (!isset($cache[$reftable][$language][$refids])) {
        $cache[$reftable][$language][$refids] = false;
      }
      return $cache[$reftable][$language][$refids];
    }
  }

  /** -translation functions */

}